==> ppcboot-0.4.2.tar.gz: ppcboot-0.4.2/common/board.c <==
/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include <command.h>

#include <mpc8xx.h>

/*
 * Begin and End of memory area for malloc(), and current "brk"
 */
static	ulong	mem_malloc_start = 0;
static	ulong	mem_malloc_end	 = 0;
static	ulong	mem_malloc_brk	 = 0;

/*
 * The Malloc area is immediately below the monitor copy in DRAM
 */
static void mem_malloc_init (ulong dest_addr)
{
	mem_malloc_end	 = dest_addr;
	mem_malloc_start = dest_addr - CFG_MALLOC_LEN;
	mem_malloc_brk = mem_malloc_start;

	memset ((void *)mem_malloc_start, 0, mem_malloc_end - mem_malloc_start);
}

void *sbrk (ptrdiff_t increment)
{
	ulong old = mem_malloc_brk;
	ulong new = old + increment;

	if ((new < mem_malloc_start) ||
	    (new > mem_malloc_end) ) {
		return (NULL);
	}
	mem_malloc_brk = new;
	return ((void *)old);
}

/*
 * Breath some life into the board...
 *
 * Set up the memory map, initialise a bunch of registers, initialise the
 * UPM's, initialise an SMC for serial comms, and carry out some hardware
 * tests.
 *
 * The first part of initialization is running from Flash memory;
 * its main purpose is to initialize the CPU and DRAM so that we
 * can relocate the monitor code to RAM.
 */
void
board_init_f (volatile immap_t *immr, unsigned long bootflag)
{
    volatile	memctl8xx_t *memctl = &immr->im_memctl;
    bd_t	*bd;
    ulong	reg, len;
    int		board_type;
    ulong	addr_moni, addr_sp;
    ulong	dram_size, cpu_speed;
    char	*s, *e;

    /* SYPCR - contains watchdog control (11-9) */

    immr->im_siu_conf.sc_sypcr = CFG_SYPCR;


    /* SIUMCR - contains debug pin configuration (11-6) */

    immr->im_siu_conf.sc_siumcr |= CFG_SIUMCR;

    /* initialize timebase status and control register (11-26) */
    /* unlock TBSCRK */

    immr->im_sitk.sitk_tbscrk = KAPWR_KEY;
    immr->im_sit.sit_tbscr = CFG_TBSCR;

    /* initialize the PIT (11-31) */

    immr->im_sitk.sitk_piscrk = KAPWR_KEY;
    immr->im_sit.sit_piscr = CFG_PISCR;

    /* PLL (CPU clock) settings (15-30) */

    immr->im_clkrstk.cark_plprcrk = KAPWR_KEY;
    reg = immr->im_clkrst.car_plprcr;
    reg &= PLPRCR_MF_MSK;		/* isolate MF field	*/
    reg |= CFG_PLPRCR;			/* reset control bits	*/
    immr->im_clkrst.car_plprcr = reg;

    /* System integration timers. Don't change EBDF! (15-27) */

    immr->im_clkrstk.cark_sccrk = KAPWR_KEY;
    reg = immr->im_clkrst.car_sccr;
    reg &= SCCR_MASK;
    reg |= CFG_SCCR;
    immr->im_clkrst.car_sccr = reg;

    /*
     * Memory Controller:
     */

    /* perform BR0 reset that MPC850 Rev. A can't guarantee */
    memctl->memc_br0 = 0x00000001;	/* just "bank valid" bit */

    /* Map banks 0 and 1 to the FLASH banks 0 and 1 at preliminary
     * addresses - these have to be modified later when FLASH size
     * has been determined
     */

    memctl->memc_or0 = CFG_OR0_REMAP;
    memctl->memc_or1 = CFG_OR1_REMAP;

    /* now restrict to preliminary range */
    memctl->memc_br0 = CFG_BR0_PRELIM;

    memctl->memc_or0 = CFG_OR0_PRELIM;
    memctl->memc_or1 = CFG_OR1_PRELIM;

    memctl->memc_br1 = CFG_BR1_PRELIM;

    /* set up serial port */
    serial_init ();

    display_options();

    printf ("Initializing...\n");

    /* CPU Clock Speed */
    cpu_speed = get_gclk_freq () / 1000 / 1000;		/* in MHz */

    printf ("  CPU:   ");		/* Check CPU		*/

    if (checkcpu(cpu_speed) < 0) {
    	printf ("*** failed ***\n");
	hang();
    }

    printf ("  Board: ");		/* Check Board		*/

    if ((board_type = checkboard()) < 0) {
    	printf ("*** failed ***\n");
	hang();
    }

    /* Map bank 4: SRAM */

    printf ("  DRAM:  ");

    if ((dram_size = initdram (board_type)) > 0) {
	printf ("%2ld MB\n", dram_size >> 20);
    } else {
	printf ("*** failed ***\n");
	hang();
    }

    /*
     * Now that we have DRAM mapped and working, we can
     * relocate the code and continue running from DRAM.
     *
     * First reserve memory for monitor code at end of DRAM.
     */
    len = get_endaddr() - CFG_FLASH_BASE;
    if (CFG_MONITOR_LEN > len)
	len = CFG_MONITOR_LEN;
    /* round up to next 4 kB limit */
    len = (len + (4096 - 1)) & ~(4096 - 1);

    addr_moni = CFG_SDRAM_BASE + dram_size - len;

    /*
     * Then we (permanently) allocate a Board Info struct.
     *
     * We leave room for the malloc() arena.
     */
    bd = (bd_t *)(addr_moni - sizeof(bd_t) - CFG_MALLOC_LEN);

    /*
     * Finally, we set up a new (bigger) stack.
     *
     * Leave some safety gap for SP, force alignment on 16 byte boundary
     */
    addr_sp  = (ulong)bd - 128;
    addr_sp &= ~0xF;

    /*
     * Save local variables to board info struct
     */

    bd->bi_memstart    = CFG_SDRAM_BASE;  /* start of  DRAM memory		*/
    bd->bi_memsize     = dram_size;	  /* size  of  DRAM memory in bytes	*/
    bd->bi_flashstart  = CFG_FLASH_BASE;  /* start of FLASH memory		*/
    bd->bi_flashsize   = 0;	 	  /* size  of FLASH memory (PRELIM)	*/
    bd->bi_flashoffset = CFG_MONITOR_LEN; /* reserved area for startup monitor	*/
    bd->bi_sramstart   = 0; /* FIXME */	  /* start of  SRAM memory		*/
    bd->bi_sramsize    = 0; /* FIXME */	  /* size  of  SRAM memory		*/
    bd->bi_immr_base   = (ulong)immr;	  /* base  of IMMR register		*/
    bd->bi_bootflags   = bootflag;	  /* boot / reboot flag (for LynxOS)	*/

    /* IP Address */
    bd->bi_ip_addr = 0;
    s = getenv ("ipaddr");
    for (reg=0; reg<4; ++reg) {
    	ulong val = s ? simple_strtoul(s, &e, 10) : 0;
	bd->bi_ip_addr <<= 8;
	bd->bi_ip_addr  |= (val & 0xFF);
	if (s)
		s = (*e) ? e+1 : e;
    }

    s = getenv ("ethaddr");
    for (reg=0; reg<6; ++reg) {
	bd->bi_enetaddr[reg] = s ? simple_strtoul(s, &e, 16) : 0;
	if (s)
		s = (*e) ? e+1 : e;
    }
    bd->bi_intfreq     = cpu_speed;		/* Internal Freq, in MHz	*/
    bd->bi_busfreq     = CONFIG_8xx_BUSCLOCK;	/* Bus Freq,      in MHz	*/

    /* Console Baudrate */
    s = getenv ("baudrate");
    bd->bi_baudrate = s ? (int)simple_strtoul(s, NULL, 10) : 9600;

    /* Function pointers must be added after code relocation */
#if 0
    bd->bi_serial_io.getc   = NULL;	/* Addr of getc() from Console	*/
    bd->bi_serial_io.tstc   = NULL;	/* Addr of tstc() from Console	*/
    bd->bi_serial_io.putc   = NULL;	/* Addr of tstc() from Console	*/
    bd->bi_serial_io.printf = NULL;	/* Addr of printf() to Console	*/

    bd->bi_interrupt.install_hdlr = NULL;
    bd->bi_interrupt.free_hdlr    = NULL;
#endif

    relocate_code (addr_sp, bd, addr_moni);

    /* NOTREACHED - relocate_code() does not return */
}

void hang(void)
{
	printf ("### ERROR ### Please RESET the board ###\n");
	for (;;);
}

void    board_init_r  (bd_t *bd, ulong dest_addr)
{
    volatile immap_t *immr = (volatile immap_t *)(bd->bi_immr_base);
    cmd_tbl_t	*cmdtp;
    ulong	flash_size;
    ulong	reloc_off = dest_addr - CFG_FLASH_BASE;

    extern void malloc_bin_reloc (ulong);
    /*
     * We have to relocate the command table manually
     */
    for (cmdtp=&cmd_tbl[0]; cmdtp->name; cmdtp++) {
	ulong addr;
	
	addr = (ulong)(cmdtp->cmd) + reloc_off;
#if 0
	printf ("Command \"%s\": 0x%08lx => 0x%08lx\n",
	    cmdtp->name, (ulong)(cmdtp->cmd), addr);
#endif
	cmdtp->cmd = (void (*)(struct cmd_tbl_s*,bd_t*,int,int,char*[]))addr;
    }

    /*
     * Setup trap handlers
     */
    trap_init(dest_addr);

    printf ("  FLASH: ");

    if ((flash_size = flash_init ()) > 0) {
	printf ("%2ld MB\n", flash_size >> 20);
    } else {
	printf ("*** failed ***\n");
	hang();
    }
    bd->bi_flashsize = flash_size;   /* size of FLASH memory (final value)	*/

    /*
     * Unlock Time Base (TBU - upper & TBL - lower) registers (11-24)
     */
    immr->im_sitk.sitk_tbk  = KAPWR_KEY;

    /* Enable Time Base so udelay() works */
    immr->im_sit.sit_tbscr |= TBSCR_TBE;

    /*
     * Enable Interrupts
     */
    interrupt_init (bd);

    udelay(20);

#if 0
  {
    volatile	memctl8xx_t *memctl = &immr->im_memctl;
    printf ("\n");
    printf ("# BR0: 0x%08x  OR0: 0x%08x\n", memctl->memc_br0, memctl->memc_or0);
    printf ("# BR1: 0x%08x  OR1: 0x%08x\n", memctl->memc_br1, memctl->memc_or1);
    printf ("# BR2: 0x%08x  OR2: 0x%08x\n", memctl->memc_br2, memctl->memc_or2);
    printf ("# BR3: 0x%08x  OR3: 0x%08x\n", memctl->memc_br3, memctl->memc_or3);
    printf ("# BR4: 0x%08x  OR4: 0x%08x\n", memctl->memc_br4, memctl->memc_or4);
    printf ("# BR5: 0x%08x  OR5: 0x%08x\n", memctl->memc_br5, memctl->memc_or5);
    printf ("# BR6: 0x%08x  OR6: 0x%08x\n", memctl->memc_br6, memctl->memc_or6);
    printf ("# BR7: 0x%08x  OR7: 0x%08x\n", memctl->memc_br7, memctl->memc_or7);
  }
#endif

    /* Insert function pointers now that we have relocated the code */

    bd->bi_serial_io.getc   = serial_getc;
    bd->bi_serial_io.tstc   = serial_tstc;
    bd->bi_serial_io.putc   = serial_putc;
    bd->bi_serial_io.printf = printf;

    bd->bi_interrupt.install_hdlr = cpm_install_handler;
    bd->bi_interrupt.free_hdlr    = cpm_free_handler;

    /* initialize malloc() area */
    mem_malloc_init (dest_addr);
    malloc_bin_reloc (reloc_off);

    /* Initialization complete - start the monitor */

    main_loop (bd);

    /* NOTREACHED - start_main does not return; just in case: loop here */
    for (;;)
	;
}

==> ppcboot-0.4.3.tar.gz: ppcboot-0.4.3/common/board.c <==
/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <ppcboot.h>
#include <command.h>

static char *failed = "*** failed ***\n";

/*
 * Begin and End of memory area for malloc(), and current "brk"
 */
static	ulong	mem_malloc_start = 0;
static	ulong	mem_malloc_end	 = 0;
static	ulong	mem_malloc_brk	 = 0;

/*
 * The Malloc area is immediately below the monitor copy in DRAM
 */
static void mem_malloc_init (ulong dest_addr)
{
	mem_malloc_end	 = dest_addr;
	mem_malloc_start = dest_addr - CFG_MALLOC_LEN;
	mem_malloc_brk = mem_malloc_start;

	memset ((void *)mem_malloc_start, 0, mem_malloc_end - mem_malloc_start);
}

void *sbrk (ptrdiff_t increment)
{
	ulong old = mem_malloc_brk;
	ulong new = old + increment;

	if ((new < mem_malloc_start) ||
	    (new > mem_malloc_end) ) {
		return (NULL);
	}
	mem_malloc_brk = new;
	return ((void *)old);
}

/*
 * Breath some life into the board...
 *
 * Initialize an SMC for serial comms, and carry out some hardware
 * tests.
 *
 * The first part of initialization is running from Flash memory;
 * its main purpose is to initialize the RAM so that we
 * can relocate the monitor code to RAM.
 */
void
board_init_f (ulong bootflag)
{
    bd_t	*bd;
    ulong	reg, len;
    int		board_type;
    ulong	addr_moni, addr_sp;
    ulong	dram_size, cpu_speed;
    char	*s, *e;

    /* CPU Clock Speed */
    cpu_speed = get_gclk_freq () / 1000 / 1000;		/* in MHz */

    /* set up serial port */
    serial_init (cpu_speed);

    display_options();

    printf ("Initializing...\n  CPU:   ");		/* Check CPU		*/

    if (checkcpu(cpu_speed) < 0) {
    	printf (failed);
	hang();
    }

    printf ("  Board: ");		/* Check Board		*/

    if ((board_type = checkboard()) < 0) {
    	printf (failed);
	hang();
    }

    /* Map bank 4: SRAM */

    printf ("  DRAM:  ");

    if ((dram_size = initdram (board_type)) > 0) {
	printf ("%2ld MB\n", dram_size >> 20);
    } else {
	printf (failed);
	hang();
    }

    /*
     * Now that we have DRAM mapped and working, we can
     * relocate the code and continue running from DRAM.
     *
     * First reserve memory for monitor code at end of DRAM.
     */
    len = get_endaddr() - CFG_FLASH_BASE;
    if (len > CFG_MONITOR_LEN) {
	printf ("*** PPCBoot size %ld > reserved memory (%d)\n",
		len, CFG_MONITOR_LEN);
	hang();
    }
    if (CFG_MONITOR_LEN > len)
	len = CFG_MONITOR_LEN;
    /* round up to next 4 kB limit */
    len = (len + (4096 - 1)) & ~(4096 - 1);

    addr_moni = CFG_SDRAM_BASE + dram_size - len;

    /*
     * Then we (permanently) allocate a Board Info struct.
     *
     * We leave room for the malloc() arena.
     */
    bd = (bd_t *)(addr_moni - sizeof(bd_t) - CFG_MALLOC_LEN);

    /*
     * Finally, we set up a new (bigger) stack.
     *
     * Leave some safety gap for SP, force alignment on 16 byte boundary
     */
    addr_sp  = (ulong)bd - 128;
    addr_sp &= ~0xF;

    /*
     * Save local variables to board info struct
     */

    bd->bi_memstart    = CFG_SDRAM_BASE;  /* start of  DRAM memory		*/
    bd->bi_memsize     = dram_size;	  /* size  of  DRAM memory in bytes	*/
    bd->bi_flashstart  = CFG_FLASH_BASE;  /* start of FLASH memory		*/
    bd->bi_flashsize   = 0;	 	  /* size  of FLASH memory (PRELIM)	*/
    bd->bi_flashoffset = CFG_MONITOR_LEN; /* reserved area for startup monitor	*/
    bd->bi_sramstart   = 0; /* FIXME */	  /* start of  SRAM memory		*/
    bd->bi_sramsize    = 0; /* FIXME */	  /* size  of  SRAM memory		*/
#ifdef	CONFIG_8xx
    bd->bi_immr_base   = CFG_IMMR;	  /* base  of IMMR register		*/
#endif
    bd->bi_bootflags   = bootflag;	  /* boot / reboot flag (for LynxOS)	*/

    /* IP Address */
    bd->bi_ip_addr = 0;
    s = getenv ("ipaddr");
    for (reg=0; reg<4; ++reg) {
    	ulong val = s ? simple_strtoul(s, &e, 10) : 0;
	bd->bi_ip_addr <<= 8;
	bd->bi_ip_addr  |= (val & 0xFF);
	if (s)
		s = (*e) ? e+1 : e;
    }

    s = getenv ("ethaddr");
    for (reg=0; reg<6; ++reg) {
	bd->bi_enetaddr[reg] = s ? simple_strtoul(s, &e, 16) : 0;
	if (s)
		s = (*e) ? e+1 : e;
    }
    bd->bi_intfreq  = cpu_speed;		/* Internal Freq, in MHz	*/
    bd->bi_busfreq  = get_bus_freq(cpu_speed);	/* Bus Freq,      in MHz	*/

    /* Console Baudrate */
    s = getenv ("baudrate");
    bd->bi_baudrate = s ? (int)simple_strtoul(s, NULL, 10) : 9600;

    /* Function pointers must be added after code relocation */
#if 0
    bd->bi_serial_io.getc   = NULL;	/* Addr of getc() from Console	*/
    bd->bi_serial_io.tstc   = NULL;	/* Addr of tstc() from Console	*/
    bd->bi_serial_io.putc   = NULL;	/* Addr of tstc() from Console	*/
    bd->bi_serial_io.printf = NULL;	/* Addr of printf() to Console	*/

    bd->bi_interrupt.install_hdlr = NULL;
    bd->bi_interrupt.free_hdlr    = NULL;
#endif

    relocate_code (addr_sp, bd, addr_moni);

    /* NOTREACHED - relocate_code() does not return */
}

void hang(void)
{
	printf ("### ERROR ### Please RESET the board ###\n");
	for (;;);
}

void    board_init_r  (bd_t *bd, ulong dest_addr)
{
    cmd_tbl_t	*cmdtp;
    ulong	flash_size;
    ulong	reloc_off = dest_addr - CFG_FLASH_BASE;

    extern void malloc_bin_reloc (ulong);
    /*
     * We have to relocate the command table manually
     */
    for (cmdtp=&cmd_tbl[0]; cmdtp->name; cmdtp++) {
	ulong addr;
	
	addr = (ulong)(cmdtp->cmd) + reloc_off;
#if 0
	printf ("Command \"%s\": 0x%08lx => 0x%08lx\n",
	    cmdtp->name, (ulong)(cmdtp->cmd), addr);
#endif
	cmdtp->cmd = (void (*)(struct cmd_tbl_s*,bd_t*,int,int,char*[]))addr;
    }

    /*
     * Setup trap handlers
     */
    trap_init(dest_addr);

    printf ("  FLASH: ");

    if ((flash_size = flash_init ()) > 0) {
	printf ("%2ld MB\n", flash_size >> 20);
    } else {
	printf (failed);
	hang();
    }
    bd->bi_flashsize = flash_size;   /* size of FLASH memory (final value)	*/

    /* initialize higher level parts of CPU like time base and timers */
    cpu_init_r (bd);

    /*
     * Enable Interrupts
     */
    interrupt_init (bd);

    udelay(20);

    /* Insert function pointers now that we have relocated the code */

    bd->bi_serial_io.getc   = serial_getc;
    bd->bi_serial_io.tstc   = serial_tstc;
    bd->bi_serial_io.putc   = serial_putc;
    bd->bi_serial_io.printf = printf;

    bd->bi_interrupt.install_hdlr = cpm_install_handler;
    bd->bi_interrupt.free_hdlr    = cpm_free_handler;

    /* initialize malloc() area */
    mem_malloc_init (dest_addr);
    malloc_bin_reloc (reloc_off);

    /* Initialization complete - start the monitor */

    main_loop (bd);

    /* NOTREACHED - start_main does not return; just in case: loop here */
    for (;;)
	;
}
